import asyncio
import logging
from logging import LogRecord
import sys
import os
from datetime import datetime

from discord import TextChannel
from discord.ext.commands import Bot as DiscordBot


__LOG_FORMAT__ = '%(asctime)s %(name)-5s %(levelname)-8s %(message)s'
__DATE_FORMAT__ = '%Y-%m-%d %H:%M:%S'


def create_logger(name: str) -> logging.Logger:
  """Create logger with some sane defaults."""
  logger = logging.getLogger(name)
  logger.setLevel(logging.DEBUG)

  stream_handler = logging.StreamHandler(stream=sys.stdout)

  COMMIT = os.getenv('GIT_COMMIT') or 'no-commit'
  timestamp = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
  log_path = f"{COMMIT}_{timestamp}.log"

  file_handler = logging.FileHandler(log_path)

  formatter = logging.Formatter(
      fmt='%(asctime)s %(name)-5s %(levelname)-8s %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
  stream_handler.setFormatter(formatter)
  file_handler.setFormatter(formatter)

  logger.addHandler(stream_handler)
  logger.addHandler(file_handler)

  return logger


class DiscordLogger(logging.Handler):
  """Logging handler which sends logs to the specified discord channel."""

  def __init__(self, bot: DiscordBot, channel_id=None, clear_on_startup=False):
    super().__init__()
    self._bot = bot
    self._channel_id = channel_id
    self._channel: TextChannel = None
    self._channel_cleared = False
    self._clear_on_startup = clear_on_startup
    self._formatter = logging.Formatter(fmt='%(asctime)s %(name)-5s %(levelname)-8s %(message)s', datefmt='%Y-%m-%d %H:%M:%S')

  def emit(self, record: LogRecord) -> None:
    if self._channel_id is not None:
      msg = self._formatter.format(record)

      async def _emit_to_channel() -> None:
        if self._channel is None:
          self._channel = await self._bot.fetch_channel(self._channel_id)
          if self._clear_on_startup:
            async for log in self._channel.history(limit=None):
              await log.delete()

        if self._channel is not None:
          await self._channel.send(msg)
      try:
        asyncio.create_task(_emit_to_channel())
      except RuntimeError:
        # This is a hotfix for "RuntimeError: no running event loop" (see #22)
        # TODO investigate #22 and replace hotfix with proper fix
        pass
